@@ -3,8 +3,9 @@ from .node import Node
 from .decorator import node
 from .executor import NodeExecutor
 from .nodes import NodePool
+from .group import group
 
-__version__ = "0.2.9"
+__version__ = "0.2.13"
 
 
-__all__ = ["NodeGraph", "Node", "node", "NodeExecutor", "NodePool"]
+__all__ = ["NodeGraph", "Node", "node", "NodeExecutor", "NodePool", "group"]
