@@ -1,9 +1,12 @@
+from __future__ import annotations
+
 from uuid import uuid1
+from node_graph.group import DependencyGroup
 from node_graph.sockets import SocketPool
 from node_graph.properties import PropertyPool
 from typing import List, Optional, Dict, Any, Union
 from node_graph.utils import deep_copy_only_dicts
-from node_graph.socket import NodeSocket, NodeSocketNamespace
+from node_graph.socket import BaseSocket, NodeSocket, NodeSocketNamespace, WaitingOn
 from node_graph_widget import NodeGraphWidget
 from node_graph.collection import (
     PropertyCollection,
@@ -84,6 +87,7 @@ class Node:
         self.create_sockets()
         self._args_data = None
         self._widget = None
+        self._waiting_on = WaitingOn(node=self, graph=self.graph)
 
     @property
     def widget(self) -> NodeGraphWidget:
@@ -291,7 +295,7 @@ class Node:
             if data["metadata"].get(key):
                 setattr(self, key, data["metadata"].get(key))
         # properties first, because the socket may be dynamic
-        for prop in data["properties"].values():
+        for prop in data.get("properties", {}).values():
             self.properties[prop["name"]].value = prop["value"]
         # inputs
         input_values = collect_values_inside_namespace(data["inputs"])
@@ -443,7 +447,6 @@ class Node:
         """Modify and save a node to database."""
 
     def to_widget_value(self):
-
         tdata = self.to_dict()
 
         for key in ("properties", "executor", "node_class", "process"):
@@ -472,3 +475,27 @@ class Node:
         """Write a standalone html file to visualize the task."""
         self.widget.value = self.to_widget_value()
         return self.widget.to_html(output=output, **kwargs)
+
+    def __rshift__(self, other: "Node" | BaseSocket | DependencyGroup):
+        """
+        Called when we do: self >> other
+        So we link them or mark that 'other' must wait for 'self'.
+        """
+        if isinstance(other, DependencyGroup):
+            for item in other.items:
+                self >> item
+        else:
+            other._waiting_on.add(self)
+        return other
+
+    def __lshift__(self, other: "Node" | BaseSocket | DependencyGroup):
+        """
+        Called when we do: self << other
+        Means the same as: other >> self
+        """
+        if isinstance(other, DependencyGroup):
+            for item in other.items:
+                self << item
+        else:
+            self._waiting_on.add(other)
+        return other
