@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+from node_graph.group import DependencyGroup
 from node_graph.property import NodeProperty
 from typing import Any, Dict, List, Optional, TYPE_CHECKING, Union
 from node_graph.collection import get_item_class, EntryPointPool
@@ -154,20 +155,28 @@ class OperatorSocketMixin:
     def __ne__(self, other):
         return self._create_operator_node(op_ne, self, other)
 
-    def __rshift__(self, other: "Node"):
+    def __rshift__(self, other: BaseSocket | Node | DependencyGroup):
         """
         Called when we do: self >> other
         So we link them or mark that 'other' must wait for 'self'.
         """
-        other._waiting_on.add(self)
+        if isinstance(other, DependencyGroup):
+            for item in other.items:
+                self >> item
+        else:
+            other._waiting_on.add(self)
         return other
 
-    def __lshift__(self, other: "Node"):
+    def __lshift__(self, other: BaseSocket | Node | DependencyGroup):
         """
         Called when we do: self << other
         Means the same as: other >> self
         """
-        self._waiting_on.add(other)
+        if isinstance(other, DependencyGroup):
+            for item in other.items:
+                self << item
+        else:
+            self._waiting_on.add(other)
         return other
 
 
@@ -176,17 +185,25 @@ class WaitingOn:
     A small helper class that manages 'waiting on' dependencies for a Socket.
     """
 
-    def __init__(self, parent: "BaseSocket") -> None:
-        self.parent = parent
-        self._waiting_on = set()
+    def __init__(self, node: "BaseSocket", graph: "NodeGraph") -> None:
+        self.node = node
+        self.graph = graph
 
-    def add(self, socket: "BaseSocket") -> None:
+    def add(self, other: "BaseSocket" | "Node") -> None:
         """Add a socket to the waiting list."""
-        if socket._node.name not in self._waiting_on:
-            self._waiting_on.add(socket._node.name)
-            self.parent._graph.add_link(
-                socket._node.outputs._wait, self.parent._node.inputs._wait
-            )
+        from node_graph.node import Node
+
+        if isinstance(other, BaseSocket):
+            node = other._node
+        elif isinstance(other, Node):
+            node = other
+        else:
+            raise TypeError(f"Expected BaseSocket or Node, got {type(other).__name__}.")
+        link_name = f"{node.name}._wait -> {self.node.name}._wait"
+        if link_name not in self.graph.links:
+            self.graph.add_link(node.outputs._wait, self.node.inputs._wait)
+        else:
+            print(f"Link {link_name} already exists, skipping creation.")
 
 
 @dataclass()
@@ -252,7 +269,7 @@ class SocketMetadata:
             )
             return cls(**known)
         raise TypeError(
-            "metadata must be dict | SocketMetadata | None – got " f"{type(raw)!r}"
+            f"metadata must be dict | SocketMetadata | None – got {type(raw)!r}"
         )
 
 
@@ -298,7 +315,7 @@ class BaseSocket:
         self._links = []
         self._link_limit = link_limit
         self._metadata: SocketMetadata = SocketMetadata.from_raw(metadata)
-        self._waiting_on = WaitingOn(self)
+        self._waiting_on = WaitingOn(node=self._node, graph=self._graph)
 
     @property
     def _full_name(self) -> str:
@@ -347,7 +364,6 @@ class BaseSocket:
 
 
 class NodeSocket(BaseSocket, OperatorSocketMixin):
-
     _identifier: str = "NodeSocket"
 
     _socket_property_class = NodeProperty
@@ -601,7 +617,6 @@ class NodeSocketNamespace(BaseSocket, OperatorSocketMixin):
         metadata: Optional[dict] = None,
         **kwargs: Any,
     ) -> object:
-
         identifier = identifier or self._SocketPool["any"]
         check_identifier_name(identifier, self._SocketPool)
 
@@ -644,7 +659,6 @@ class NodeSocketNamespace(BaseSocket, OperatorSocketMixin):
 
     @property
     def _value(self) -> Dict[str, Any]:
-
         data = {}
         for name, item in self._sockets.items():
             if isinstance(item, NodeSocketNamespace):
@@ -855,4 +869,4 @@ Acceptable names are {self._get_keys()}. This collection belongs to {self._paren
 
     def __repr__(self) -> str:
         nested = list(self._sockets.keys())
-        return f"{self.__class__.__name__}(name='{self._name}', " f"sockets={nested})"
+        return f"{self.__class__.__name__}(name='{self._name}', sockets={nested})"
